function [o] = unemploymentAndStocks(H,method,suppress)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes dynamic H-step ahead forecasts for a VAR
% and plots dynamic forecasts for unemployment
% o is an output structure
% Set method == 1 for Vecm comparison
% Default is bivariate Var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize
o=[];
o.pv = setParamsVecm;
o.pu = setParamsUnivar;
o.pV = setParamsVar;
disp('Estimation Sample is 1953.1:1979.3');

if nargin < 2  || isempty(method)
    method = 1;
    disp('Comparison method is bivariate Var');
end

if nargin <3
    suppress = 0;
end

if method ~= 1
    o.pv = o.pV;
elseif method == 1
    disp('Comparison methood is Vecm');
end



% Load Data
load mydat ptr utr;
o.x = [utr ptr];

% Set initial observation
o.n1 = 13;              % Corresponds to 1953q1
o.n2 = size(o.x,1);     % Sets last observation
o.x = o.x(o.n1:end,:);  % Sets Sample to post-Korean war
o.tm = (1953.1:0.25:2011.25)';


% Set the date for the forecast period
startForecastDate = '801';
N1 = findDateNumber(startForecastDate);
N2 = findDateNumber('111');
o.smpl = N1:N2;                         % Sample for forecasts
o.smplP = findDateNumber('041'):N2;     % Sample for graphs




% Choose dates to begin dynamic forecasts
% Default is 2008q1, 2009q1 and 2010q1
% Last date is set by picking a date string for tt
% If supress = 1, no dynamic forecasts are made
if ~ suppress
    tt = findDateNumber('091');
    sp = 1;
    for j = [tt-8 tt-4 tt];
        
        o.xfv1 = dynamicForecast(j,o.x,o.pv);
        o.xfu1 = dynamicForecast(j,o.x,o.pu);
        
        % Construct plots for dynamic forecasts
        
        figure(sp);
        hold on;
        plot(o.tm(o.smplP),transform(o.xfv1(o.smplP,1)),'r-x' ,'Linewidth',2.5);
        hold on;
        plot(o.tm(o.smplP), transform(o.xfu1(o.smplP,1)),'k-o','Linewidth',2.5);
        plot(o.tm(o.smplP), transform(o.x(o.smplP)),'Linewidth',2.5);
        h_legend = legend('A Forecast Using the Stock Market','A Forecast without the Stock Market','Unemployment Data','Location','SouthEastOutside');
        
        set(h_legend,'FontSize',14);
        xlabel('Unemployment and Two Dynamic Forecasts','Fontsize',20);

        

        
        
        sp=sp+1;
        
        
    end
end

o.xfv = HstepForecast(N1,H,o.x,o.pv);
o.xfu = HstepForecast(N1,H,o.x,o.pu);
o.xfvE = o.xfv(o.smpl,:) - o.x(o.smpl,:);
o.xfuE = o.xfu(o.smpl,:) - o.x(o.smpl,:);
[o.fuUE ,o.xfuUE]=ksdensity(o.xfuE(:,1),'npoints',100);%,'width',0.1);         %  univariate
[o.fvUE ,o.xfvUE]=ksdensity(o.xfvE(:,1),'npoints',100);%,'width',0.1);         %  comparison
[o.fuPE ,o.xfuPE]=ksdensity(o.xfuE(:,2),'npoints',100);%,'width',0.1);         %  comparison
[o.fvPE ,o.xfvPE]=ksdensity(o.xfvE(:,2),'npoints',100);%,'width',0.1);         %  vecm


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% END OF MAIN PROGRAM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function p = setParamsVecm
        % Function to set parametes of Vecm
        % Estiates were computed in Eviews using Vecm procedure
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        p.DA1 = [0.596752017117, - 0.271052218993;...
            0.130881299955, 0.417198917668];
        p.Beta = [1; 0.623053093979;  - 7.44163676558];
        p.Alpha = [ - 0.0997624371173 ; 0.00871188402458];
        
        
        p.Pi = p.Alpha*p.Beta';
        p.A1 = eye(2) + p.DA1 + p.Pi(:,1:2);
        p.A2 = -p.DA1;
        p.C = p.Pi(:,3);
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function p = setParamsUnivar
        % Function to set parametes of Univariate model
        % Estimates were computed in Eviews using OLS on a model in first
        % differences.  Sample was 1953.1 to 1979.3
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        p.A1 = [1.63, 0; 0, 1.37 ];
        p.A2 = [-0.63, 0; 0, -0.37];
        p.C =  [ 0 ; 0  ];   % Set insignificant constants to zero
        
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function p = setParamsVar
        % Function to set parametes of Bivariate Var
        % Equations estimated in differences by robustleast squares
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         p.DA1 = [0.596752017117, - 0.3182;...
%             0.1, 0.38];
            p.DA1 = [0.49, - 0.36;...
            0.13, 0.40];
        p.A1 = eye(2) + p.DA1 ;
        p.A2 = -p.DA1;
        p.C = [0;0];
        
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function xf = dynamicForecast(T,x,p)
        % Function to compute a dynamic forecast of a VAR parameterized by
        % p using data vec x
        xf =x';
        n2 = size(x,1);
        for t = T:n2
            xf(:,t) = p.C + p.A1*xf(:,t-1) + p.A2*xf(:,t-2) ;
        end
        xf = xf';
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function xf = HstepForecast(T,H,x,p)
        % Function to compute an H-step ahead forecast of a VAR
        % parameterized by p using data vec x. T is the beginning of the
        % forecast period.
        xf = x;
        for ss = T:length(x)-H
            xftemp = dynamicForecast(ss,x,p);
            xf(ss+H,:) = xftemp(ss+H,:);
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function N = findDateNumber(nStr)
        % Takes a 3 digit string. First two digits are year, third is
        % quarter.  Returns number of quarters since 1953.1
        ynum = str2double(nStr(1:2));
        ynum = (ynum>=53)*(ynum - 53) + (ynum<53)*(47+ynum);
        qnum = str2double(nStr(3));
        N = ynum*4 + qnum;
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function ut = transform(u)
        % Converts transformed unemployment back to levels
        ut = 100.*exp(u)./(100+exp(u));
        
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% END OF SUBROUTINES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
